#!/bin/sh
#
# Copyright (C) 2003, 2005 InitZero S.rl.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#

save_output () {
  tee "${file_log_output}"
}

###@@@
###@@@ text report
###@@@
report_text() {
  local type=$1

  case ${type} in
	system)			msg "             System: " ; echo "${system}" ;;
	method)			msg "             Method: " ; echo "${method}" ;;
	method_option)		msg "             Option: " ; echo "${method_option}" ;;
	method_type)		msg "               Type: " ; echo "${method_type}" ;;
	backup_source)		msg "             Source: " ; echo "${backup_source}" ;;
	backup_remote_uri)	msg "         Remote URI: " ; echo "${backup_remote_uri}" ;;
	backup_step)		msg "         Backing up: " ; echo -n "'${backup_source_step}' " ;;
	backup_destination)	msg "        Destination: " ; echo "${backup_destination}" ;;
	backup_destination_step)msg "   Real destination: " ; echo "${backup_destination_step}" ;;
	backup_exclude)		msg "          Exclusion: " ; echo "${backup_exclude}" ;;
	backup_incremental)	msg "        Incremental: " ; echo "${backup_incremental}" ;;
	backup_erase_after)	msg "        Erase after: " ; echo -n "${backup_erase_after} " ; msg "backups" ; echo ;;
	backup_verify)		msg "    Veryfing backup: " ; echo -e "\n";;
	synbak_version)		msg "     Synbak version: " ; echo "${synbak_version} - ${synbak_version_date}" ;;
	synbak_host)		msg "      Synbak server: " ; echo "${synbak_server}" ;;
	technical_support)	msg "  Technical support: " ; echo "${report_info_support}" ;;
	time_start)		msg "       Backup begin: " ; echo "$(show_time_human ${time_begin} full)" ;;
	time_end)		msg "         Backup end: " ; echo "$(show_time_human ${time_end} full)" ;;
	time_duration)		msg "    Backup duration: " ; echo "$(show_time_duration_human ${time_begin} ${time_end})" ;;
	backup_erase)		msg "    Keeping backups: " ; echo "'$(cat ${file_list_backup_keep} | tr -s "\n" " " | sed 's/^ *//g' | sed 's/ *$//g')'"
				msg "    Erasing backups: " ; echo -n "'$(cat ${file_list_backup_erase})' " ;;
	load_tape_step)		msg "       Loading tape: number " ; echo -n "'${tape}' " ; msg "on" ; echo -n " '${backup_device_changer}' " ;;
	unload_tape_step)	msg "     Unloading tape: from " ; echo -n "'${backup_device_changer}' " ;;
	eject_device_step)	msg "    Ejecting device: " ; echo -n "'${backup_destination}' " ;;
	blank_device_step)	msg "    Blanking device: " ; echo -n "'${backup_destination}' " ;;
	mount_step)		msg "           Mounting: " ; echo -n "'${backup_remote_uri}' " ; msg "on" ; echo -n " '${dir_mnt}' " ;;
	umount_step)		msg "         Unmounting: " ; echo -n "'${dir_mnt}' " ;;
	files_total)		msg " Total source files: " ; echo "$(show_files ${files_total})" ;;
	files_backup)		msg "       Backup files: " ; echo "$(show_files ${files_backup})" ;;
	size_backup)		msg "        Backup size: " ; echo "$(show_size ${size_backup})" ;;
	size_destination)	msg "   Destination size: " ; echo "$(show_size ${size_destination})" ;;
	size_source)		msg "        Source size: " ; echo "$(show_size ${size_source})" ;;
	speed_backup)		msg "       Backup speed: " ; echo "$(show_speed ${size_backup} ${time_begin} ${time_end})" ;;
	report_step)		msg "  Generating report: " ; echo -n "'$(basename $report)' " ;;
	status_backup)		msg "      Backup result: " ; echo "$(check_status_backup)" ;;
	info_files)		msg "Files:"    ; echo -n "[$(show_files ${files_backup})] " ;;
	info_size)		msg "Size:"     ; echo -n "[$(show_size ${size_backup})] " ;;
	info_speed)		msg "Speed:"    ; echo -n "[$(show_speed ${size_backup} ${time_step_begin} ${time_step_end})] " ;;
	info_duration)		msg "Duration:" ; echo -n "[$(show_time_duration_human ${time_step_begin} ${time_step_end})] ";;
	info_status)		msg "Status:"   ; echo -n "[${step_status}] ";;
	backup_step_info)	[ -n "${files_backup}" ] && report_text info_files ; report_text info_size ; report_text info_speed ; report_text info_duration ; report_text info_status ; echo ;;
	step_info)		report_text info_duration ; report_text info_status ; echo ;;
	step_info_error)	msg "ERROR: An error is occurred in this step, follow error details:" >&2 ;;
	status_host_down)	msg "ERROR: The remote host '\${backup_remote_host}' is down" ; echo ;;
	dir_not_exist)		msg "ERROR: '\$dir' directory doesn't exist or it isn't accessible" >&2 ; echo >&2 ;;
	file_not_exist)		msg "ERROR: '\$file' file doesn't exist or it isn't accessible" >&2 ; echo >&2 ;;
	device_not_exist)	msg "ERROR: '\$file' file doesn't exist, isn't writable or it's not a character device" >&2 ; echo >&2 ;;
	field_empty)		msg "ERROR: in file '\${usr_file_conf}' the mandatory '\${field}' field is empty" >&2 ; echo >&2 ;;
	field_wrong)		msg "ERROR: in file '\${usr_file_conf}' the field '\${field}' contains invalid data: '\${value}'" >&2 ; echo >&2 ;;
	backup_erase_error)	msg "ERROR: Not erasing old backups because an ERROR is occurred" ;;
	uri_unsupported)	msg "ERROR: The specified remote URI '\${backup_remote_uri}' is unsupported" ; echo ;;
	protocol_unsupported)	msg "ERROR: The specified remote protocol '\${backup_remote_protocol}' is unsupported" ; echo ;;
	mount_unsupported)	msg "ERROR: Unsupported remote mount share" ; echo ;;
	extra_option_required)	msg "ERROR: Extra options are required, follow a list of usable extra options: \$extra_option_right" ; echo ;;
	extra_option_wrong)	msg "ERROR: The specified extra option '\$current_option' is invalid, follow a list of usable extra options: \$extra_option_right" ; echo ;;
	internal_error)		msg "ERROR: Synbak internal error, please report this to developers" ; echo ;;
	ok)			msg "OK" ;;
	error)			msg "ERROR" ;;
	unknow)			msg "unknow" ;;
	separator_info1)	echo "@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@" ;;
	separator_info2)	echo "================================================================================" ;;
	separator_info3)	echo "--------------------------------------------------------------------------------" ;;
	separator_info4)	echo "~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~"	;;
	separator_stats)	echo "................................................................................"	;;
	separator_error)	echo "********************************************************************************"	;;
	system_info)
				msg "Backup informations: " ; echo
				report_text separator_stats
				msg "Directory list of " ; echo "'${backup_destination}':"
				ls -Al "${backup_destination}" | grep -v ^total
				report_text separator_stats
				msg "Active mount points: " ; echo
				df -Tmh -x tmpfs
				report_text separator_stats
				;;
esac
}

# desc: show a variable value by providing only a variable name
# args: 'variable name'
show_variable_value() {
  eval local varval=\$$1
  echo "$varval"
}


# function description: parse and import the user config file fields
# args options: $1 $2 $3 $4 
# args description: file_config_field_import 'user file config location' 'field name' 'field type: bolean, integer, text' 'null, notnull'
file_config_field_import() {
  if [[ -z "$1" || -z "$2" || -z "$3" || -z "$4" ]]; then echo "$(report_text internal_error) \$@=$@"; exit 1; fi

  local file_config=$1
  local field=$2
  local type=$3
  local null=$4

  # automatically create a variable named like the field found in the config file
  eval $field="\"$(cat ${file_config} 		| \
			grep -v ^"#"	 	| \
			grep -w ^${field} 	| \
			tail -n 1 | grep "=" 	| \
			sed 's/=/==AWKFIELD==/'	| \
			awk -F "==AWKFIELD==" '{print $2}' | \
			awk -F "#" '{print $1}' | \
			tr -d [:cntrl:] 	| \
			sed 's/\"//g' 		| \
			sed 's/^ *//g' 		| \
			sed 's/ *$//g'		)\""


  # verify if the imported field value is valid
  file_config_field_check ${field} ${type} ${null} $(show_variable_value ${field})
  save_status_config_field_import

  # exit if the field value is not valid
  [ $status_config_field_import -ne 0 ] && exit 1;
}


# function description: check if a variable is empty or invalid, return 1 if the check goes wrong
# args options: $1 $2 $3 $4 
# args: file_config_field_check 'check type' 'null' 'field name' 'field value'
file_config_field_check() {
  local field=$1
  shift
  local type=$1
  shift
  local null=$1
  shift
  local value=$@

  #echo type: $type
  #echo field: $field
  #echo value: $value

  case ${type} in
    bolean)
      if [ -z "${value}" ] && [ "${null}" != "null"  ]
        then
          save_status_backup_error
          report_text field_empty
          return 1;
      elif [ -z ${value} ] || [ ${value} = yes ] || [ ${value} = no ] 
        then
          return 0
        else
          save_status_backup_error
          report_text field_wrong
          return 1
      fi
      ;;
    text)
      #echo -e ${field}: ${value} - ${null}
      if [ -z "${value}" ] && [ "${null}" != "null"  ]
        then
          save_status_backup_error
          report_text field_empty
          return 1
        else
          return 0
      fi
      ;;
    path)
      #echo -e ${field}: ${value} - ${null}
      if [ -z "${value}" ] && [ "${null}" != "null"  ]
        then
          save_status_backup_error
          report_text field_empty
          return 1
        else
          return 0
      fi
      ;;
    uri)
      #echo -e ${field}: ${value} - ${null}
      if [ -z "${value}" ] && [ "${null}" != "null"  ]
        then
          save_status_backup_error
          report_text field_empty
          return 1
        else
          parse_uri "${value}"
          [ $? -eq 1 ] && report_text field_wrong && parse_uri_unset_variables && return 1
          parse_uri_unset_variables
          return 0
      fi
      ;;
    integer)
      if [ -z "${value}" ] && [ "${null}" != "null"  ]
        then
          save_status_backup_error
          report_text field_empty
          return 1;
        else
          [ -z "${value}" ] && value="0"
          echo ${value} | grep [^0-9] > /dev/null 2>&1
          if [ $? -eq 0 ] || [ ${value} -lt 0 ]
            then
              save_status_backup_error
              report_text field_wrong
              return 1
            else
              return 0
          fi
      fi
      ;;
  esac
}

# import all default fields from user config file
usr_config_field_import_default() {
  file_config_field_import "${usr_file_conf}" backup_source		path	null
  file_config_field_import "${usr_file_conf}" backup_destination	path	notnull
  file_config_field_import "${usr_file_conf}" backup_exclude		path	null

  file_config_field_import "${usr_file_conf}" backup_incremental	bolean	notnull
  file_config_field_import "${usr_file_conf}" backup_erase_after	integer	notnull
  file_config_field_import "${usr_file_conf}" backup_no_make_destdir	bolean	null
  file_config_field_import "${usr_file_conf}" backup_erase_always	bolean	notnull
  file_config_field_import "${usr_file_conf}" backup_method_opts	text	null

  file_config_field_import "${usr_file_conf}" report_info_support 	text	null

  file_config_field_import "${usr_file_conf}" report_stdout		bolean	notnull
  file_config_field_import "${usr_file_conf}" report_stdout_on_errors	bolean	notnull

  file_config_field_import "${usr_file_conf}" backup_device_changer	text	null
  file_config_field_import "${usr_file_conf}" backup_device_tape_number	integer	null
  file_config_field_import "${usr_file_conf}" backup_device_eject	bolean	null
  file_config_field_import "${usr_file_conf}" backup_verify		bolean	null

  file_config_field_import "${usr_file_conf}" backup_remote_uri		uri     null
  file_config_field_import "${usr_file_conf}" report_remote_uri_down	bolean  null

  file_config_field_import "${usr_file_conf}" report_email		bolean	null
  file_config_field_import "${usr_file_conf}" report_email_on_errors	bolean	null
  file_config_field_import "${usr_file_conf}" report_email_rcpt		text	null

  file_config_field_import "${usr_file_conf}" report_html		bolean	null
  file_config_field_import "${usr_file_conf}" report_html_on_errors	bolean  null
  file_config_field_import "${usr_file_conf}" report_html_destination	path	null
  file_config_field_import "${usr_file_conf}" report_html_logo		bolean	null
  file_config_field_import "${usr_file_conf}" report_html_logo_image	text	null
  file_config_field_import "${usr_file_conf}" report_html_logo_link	uri	null

  #[[ "${report_remote_uri_down}" = "no" && "${report_stdout}" = "yes" ]] && report_remote_uri_down=yes
  check_writable ${backup_destination} ; [ $? -ne 0 ] && exit 1; # check if the destination directory or device exist
  if [ -n "${report_html_destination}" ]; then check_writable ${report_html_destination} ; [ $? -ne 0 ] && exit 1; fi # check if the destination html report directory or device exist

  parse_path "${backup_source}"		> "${file_list_source}"
  parse_path "${backup_destination}"	> "${file_list_destination}"
  parse_path "${backup_exclude}"	| sed 's/\\//g' > "${file_list_exclude}"
}

# create a formatted file text containing the specified paths
parse_path(){
    local string=$@

    echo "${string}" | sed 's/\\ /%20/g' | awk 'BEGIN { RS = " " } {print $0}' | sed 's/%20/\\ /g' | grep -v "^$"

    ## to import the output of this command in while loop use the following command
    #cat "${filenamevar}" | while read -r backup_source_step
    #  some_command
    #done
}

  # Check if a directory exist and it's accessible
  # Return Codes:
  # 0 = Directory exist, it is writable and executable
  # 1 = Directory doesn't exist or it isn't writable/executable
check_dir() {
  if [[ -d "$1" && -r "$1" && -x "$1" ]]
    then
      return 0;
    else
      local dir=$1
      report_text dir_not_exist
      return 1;
  fi
}

  # Check if a directory, file or device exist and it's writable
  # Return Codes:
  # 0 = Directory exist, it is writable and executable
  # 1 = Directory doesn't exist or it isn't writable/executable
check_writable() {
  if [[ -e "$1" && -r "$1" && -w "$1" ]]
    then
      return 0;
    else
      local dir=$1
      report_text dir_not_exist
      return 1;
  fi
}

  # Check if a file exist and it's accessible
  # Return Codes:
  # 0 = Directory exist, it is writable and executable
  # 1 = Directory doesn't exist or it isn't writable/executable
check_file() {
  if [[ -e "$1" && -r "$1" ]]
    then
      return 0;
    else
      local file=$1
      report_text file_not_exist
      return 1;
  fi
}

check_device() {
  if [[ -c "$1" && -w "$1" ]] || [[ -b "$1" && -w "$1" ]]
    then
      return 0;
    else
      local file=$1
      report_text device_not_exist
      return 1;
  fi
}


show_available_methods() {
  for method in $(ls $sys_dir_method/)
    do
      echo $(basename $method) >&2
    done
}


show_available_reports() {
  for report in $(ls $sys_dir_report/)
    do
      echo $(basename $report) >&2
    done
}


show_backup_errors_step() {
  cat "${file_log_errors_step}" | grep -v -f "${sys_file_method_nonerrors_strings}"
}

save_status_backup_error(){
      echo "1" > "${file_status_backup}"
}

save_status_backup_ok(){
      echo "0" > "${file_status_backup}"
}

# Check the status of last backup step
check_status_backup_step() {
  if [ -n "$(show_backup_errors_step)" ]
    then
      save_status_backup_error
      local step_status="$(report_text error)"
      report_text backup_step_info
      echo
      report_text step_info_error
      echo
      report_text separator_error
      show_backup_errors_step
      report_text separator_error 

      # Make a backup of errors file
      #cp "${file_log_errors}" "${file_log_errors}.$(show_time_unix)"
      #touch "${file_log_errors}"
      return 1;
    else
      local step_status="$(report_text ok)"
      report_text backup_step_info

      # Make a backup of errors file
      #cp "${file_log_errors}" "${file_log_errors}.$(show_time_unix)"
      #touch "${file_log_errors}"
      return 0;
  fi
}

# Check the status of last step
check_status_step() {
  if [ -n "$(show_backup_errors_step)" ]
    then
      save_status_backup_error
      local step_status="$(report_text error)"
      report_text step_info
      echo
      report_text step_info_error
      echo
      report_text separator_error
      show_backup_errors_step
      report_text separator_error 

      # Make a backup of errors file
      #cp "${file_log_errors}" "${file_log_errors}.$(show_time_unix)"
      #touch "${file_log_errors}"
      return 1;
    else
      local step_status="$(report_text ok)"
      report_text step_info

      # Make a backup of errors file
      #cp "${file_log_errors}" "${file_log_errors}.$(show_time_unix)"
      #touch "${file_log_errors}"
      return 0;
  fi
  # Make a backup of errors file
  #cp "${file_log_errors}" "${file_log_errors}.$(show_time_unix)"
  #touch "${file_log_errors}"
}

# Check the final backup status
check_status_backup() {
  if [ "$(cat ${file_status_backup})" = "1" ]
    then
      report_text error
      echo
      return 1;
    else
      save_status_backup_ok
      report_text ok
      echo
      return 0;
  fi
}

save_method_info() {
  [ -n "${method_option}" ] && echo "${method_option}" > "${file_method_option}"
  [ -n "${method_type}" ]   && echo "${method_type}"   > "${file_method_type}"
}


# Date calc functions
save_time_begin() {
  time_begin=$(show_time_unix)
  echo ${time_begin} > ${file_time_begin}
}

save_time_end() {
  time_end=$(show_time_unix)
  echo ${time_end} > ${file_time_end}
}

save_time_duration() {
  time_duration=$(expr ${time_end} - ${time_begin})
}

save_time_begin_step() {
  time_step_begin=$(show_time_unix)
  RANDOM_NUMBER="${RANDOM}"
  file_log_errors_step="${file_log_errors}.${time_step_begin}.${RANDOM_NUMBER}"
  file_log_stats_step="${file_log_stats}.${time_step_begin}.${RANDOM_NUMBER}"
  touch ${file_log_errors_step} ${file_log_stats_step}
}

save_time_end_step() {
  time_step_end=$(show_time_unix)
}

# Show the current time in the unix format
show_time_unix() {
 date +%s
}

save_status_host() {
  status_host="$?"
  echo "${status_host}" > "${file_status_host}"
  #echo status_host=$status_host
}

save_status_config_field_import() {
  status_config_field_import=$?
  echo "${status_config_field_import}" > "${file_status_config_field_import}"
}

save_status_mount() {
  status_mount="$?"
  #echo ${status_mount} >${file_status_mount}
  #echo status_mount=$status_mount
}

# args: seconds in unix date format
# return: human readable date format
show_time_human(){
 local time_unix=$1
 local type=$2

 [ -n "$(echo "${time_unix}" | grep [^0-9])" ] && report_text unknow && return 1

 case ${type} in
	backup)	date --date="1970-01-01 00:00:00 UTC ${time_unix} seconds" +"%Y%m%d-%H%M%S" ;;
	full)	date --date="1970-01-01 00:00:00 UTC ${time_unix} seconds" +"%Y/%m/%d %H:%M:%S %Z" ;;
	fullnew)date --date=@${time_human} +'%Y/%m/%d %H:%M:%S %Z' ;;
	date)	date --date="1970-01-01 00:00:00 UTC ${time_unix} seconds" +"%Y%m%d" ;;
	year)	date --date="1970-01-01 00:00:00 UTC ${time_unix} seconds" +"%Y" ;;
	month)	date --date="1970-01-01 00:00:00 UTC ${time_unix} seconds" +"%m" ;;
	day)	date --date="1970-01-01 00:00:00 UTC ${time_unix} seconds" +"%d" ;;
	hour)	date --date="1970-01-01 00:00:00 UTC ${time_unix} seconds" +"%H" ;;
	minute)	date --date="1970-01-01 00:00:00 UTC ${time_unix} seconds" +"%M" ;;
	second)	date --date="1970-01-01 00:00:00 UTC ${time_unix} seconds" +"%S" ;;
	rss)	LANG_ORIG=$LANG ; LANG=en ; date --date="1970-01-01 00:00:00 UTC ${time_unix} seconds" +"%a, %e %b %Y %H:%M:%S %Z" ; LANG=$LANG_ORIG ;;
	*|time)	date --date="1970-01-01 00:00:00 UTC ${time_unix} seconds" +"%H:%M:%S" ;;
 esac
}

# args: "start time in seconds from UNIX epoch" "End Time in seconds from UNIX epoch"
# return: echoed HH:MM:SS date format
show_time_duration_human() {

 [[ -n "$(echo "$1" | grep [^0-9])" || -n "$(echo "$2" | grep [^0-9])" ]] && report_text unknow && return 1

 local time_duration_seconds="$(expr $2 - $1)"

 if [ ${time_duration_seconds} -ge 86400 ]
   then
     days="$(echo "scale=0; ${time_duration_seconds} / 86400" | bc)"
     date -u --date="1970-01-01 00:00:00 UTC ${time_duration_seconds} seconds" +"${days} day %H:%M:%S"
   else
     date -u --date="1970-01-01 00:00:00 UTC ${time_duration_seconds} seconds" +"%H:%M:%S"
 fi
}


# input: integer
# output: integer or error reached
show_files() {
 local files=$1
 [ -z "${files}" ] || [ -n "$(echo "${files}" | grep [^0-9])" ] && report_text unknow && return 1 || echo ${files}
}


# input: backup size in bytes
# output: backup size in the more human form
show_size() {
 local fsize=$1

 # 1 MB   = 1024000 Bytes
 # 10 MB  = 10240000 Bytes
 # 100 MB = 102400000 Bytes
 # 1 GB   = 1024000000 Bytes
 # 10 GB  = 10240000000 Bytes
 # 100 GB = 102400000000 Bytes
 # 1 TB   = 1024000000000 Bytes
 # 10 TB  = 10240000000000 Bytes
 # 100 TB = 102400000000000 Bytes

 #echo fsize=\"$fsize\" ftime_begin=\"$ftime_begin\" ftime_end=\"$ftime_end\"

 [ -z "${fsize}" ] || [ -n "$(echo "${fsize}" | grep [^0-9])" ] && report_text unknow && return 1

 [ "${fsize}" -lt 1024  ] && echo "${fsize} B" && return 1

  if   [ ${fsize} -ge 1024000000000 ]
    then
      echo $(echo "scale=2; ${fsize} / 1024000000000" | bc) TB
  elif [ ${fsize} -ge 1024000000 ]
    then
      echo $(echo "scale=2; ${fsize} / 1024000000" | bc) GB
  elif [ ${fsize} -ge 1024000 ]
    then
      echo $(echo "scale=2; ${fsize} / 1024000" | bc) MB
  elif [ ${fsize} -ge 1024 ]
    then
      echo $(echo "scale=2; ${fsize} / 1024" | bc) KB
    else
      echo ${fsize} B
  fi
}

# input $1: backup size in bytes
# input $2: unix time of backup begin
# input $3: unix time of backup end
# output: the speed in the more human form
show_speed(){
 local fsize=$1
 local ftime_begin=$2
 local ftime_end=$3

 #echo fsize=\"$fsize\" ftime_begin=\"$ftime_begin\" ftime_end=\"$ftime_end\"

 [ -z "${fsize}" ] || [ -z "${ftime_begin}" ] || [ -z "${ftime_end}" ] || [ -n "$(echo "${fsize}" | grep [^0-9])" ] || [ -n "$(echo "${ftime_begin}" | grep [^0-9])" ] || [ -n "$(echo "${ftime_end}" | grep [^0-9])" ] && report_text unknow && return 1
 
 [ "${fsize}" -lt 1024  ] && echo "${fsize} B/Sec" && return 1
 [ "${ftime_end}" -lt "${ftime_begin}"  ] && report_text unknow && return 1

 local fduration=$(expr ${ftime_end} - ${ftime_begin})

 [ "${ftime_end}" -le "${ftime_begin}"  ] && fduration=1

 local fsize_sec=$(echo $(echo "${fsize} / ${fduration}" | bc))

  if   [ ${fsize_sec} -ge 1024000000000 ]
    then
      echo $(echo "scale=2; ${fsize_sec} / 1024000000000" | bc) TB/Sec
  elif [ ${fsize_sec} -ge 1024000000 ]
    then
      echo $(echo "scale=2; ${fsize_sec} / 1024000000" | bc) GB/Sec
  elif [ ${fsize_sec} -ge 1024000 ]
    then
      echo $(echo "scale=2; ${fsize_sec} / 1024000" | bc) MB/Sec
  elif [ ${fsize_sec} -ge 1024 ]
    then
      echo $(echo "scale=2; ${fsize_sec} / 1024" | bc) KB/Sec
    else
      echo ${fsize_sec} B/Sec
  fi
}


save_size_destination() {
  local destination="$1"

  check_file ${destination} 2>/dev/null
  if [ $? -eq 0 ]
   then 
     size_destination=$(du -csxb ${destination} | tail -n 1 | awk '{print $1}')
     echo "${size_destination}" > "${file_size_destination}"
   else
     size_destination="$(report_text unknow)"
     echo "${size_destination}" > "${file_size_destination}"
  fi
}

save_size_source() {
  local source=$1

  check_file ${source} 2>/dev/null
  if [ $? -eq 0 ]
   then 
     size_source=$(du -csxb ${source} | tail -n 1 | awk '{print $1}')
     echo ${size_source} > ${file_size_source}
   else
     size_source="$(report_text unknow)"
     echo ${size_source} > ${file_size_source}
  fi
}


# function desc: Save the backup size
save_size_backup() {
  local value=$1
  local option=$2
  local value_old=$(cat ${file_size_backup})

  case $option in
	varreplace)	size_backup="${value}"		; echo "$(count_integers)"	> "${file_size_backup}" ;; # display current backup_size but save the sum of old backups size
	allreplace)	size_backup="${value}"		; echo "${value}"		> "${file_size_backup}" ;; # display current backup_size and don't sum old backups size
	*)		size_backup="$(count_integers)"	; echo "${size_backup}"		> "${file_size_backup}" ;; # display and save the sum of old backups size variables
  esac
}


# function desc: Summarize integers numbers (value_old + value new)
count_integers(){
  if [ -n "${value_old}" ] && [ -z "$(echo "${value}" | grep [^0-9])" ]
    then
      if [ -z "${value}" ]
        then
          echo "${value_old}"
        else
          echo "$(expr ${value_old} + ${value})"
      fi
    else
      if [ -z "$(echo "${value}" | grep [^0-9])" ]
        then
          echo "${value}"
        else
          report_text unknow
          return 1
      fi
  fi
}

set_stats_backup() {
  #touch "${file_size_backup}" "${file_stats_files_backup}" "${file_stats_files_total}"
  size_backup=$(cat "${file_size_backup}")
  files_backup=$(cat "${file_stats_files_backup}")
  files_total=$(cat "${file_stats_files_total}")
}

save_stats_backup(){
  local type="$1"
  shift
  local file_stats="$1"
  shift
  local file_output="$1"
  shift
  local string=$@

  [ ! -f "${file_stats}" ]  && report_text internal_error && return 1
  [ ! -f "${file_output}" ] && report_text internal_error && return 1

  case $type in
	files_backup|files_total|size_backup)
		local value=$(cat ${file_stats} | grep ^"${string}" | awk -F: '{print $2}' | awk '{print $1}' | tr -d [:alpha:] | tr -d [:blank:] | tr -d [:cntrl:] | tr -d [:punct:])
		local value_old=$(cat $file_output)

                count_integers > "${file_output}"
                eval $type=\""${value}"\"

		# Make a backup of stats file
                #cp "${file_stats}" "${file_stats}.$(show_time_unix)"
		;;
	*)	report_text internal_error ;;
  esac
}


find_socket_files() {
  find $1 -type s 2>/dev/null
}


backup_erase_init () {
    local pattern="$1"
    previous_dir="${PWD}"
    cd "${backup_destination}"
    find -maxdepth 1 -name "${pattern}" | sed -e 's/^.\///' | grep -wv ^"." | sort 1> "${file_list_backup}"

    # be sure to use the default value of backup_erase_after variable
    #file_config_field_import ${usr_file_conf} backup_erase_after integer notnull

    # increment by one the value to be sure to keep the current day backup if 
    [ "${backup_erase_after}" -eq "0" ] && let backup_erase_after_step="${backup_erase_after}+1" || backup_erase_after_step="${backup_erase_after}"

    # make a list of all backups stored in the backup_destination directory
    cat "${file_list_backup}" | tail -n "${backup_erase_after_step}" > "${file_list_backup_keep}"

    # generate the remove list
    diff "${file_list_backup}" "${file_list_backup_keep}" | grep ^"<" | awk '{print $2}' | tr -s "\n" " " | sed 's/^ *//g' | sed 's/ *$//g' > "${file_list_backup_erase}"

    # deprecated method using find
    #for backup in $(cat ${file_list_backup_keep})
    #  do
    #    echo "! -name $backup" >> ${file_list_backup_keep_find}
    #  done

    unset backup_erase_after_step
    cd "${previous_dir}"
}

backup_erase() {
  previous_dir="${PWD}"
  cd "${backup_destination}"
  check_status_backup 2>&1 > /dev/null
  if [[ $? -eq 0 || "${backup_erase_always}" = "yes" ]] && [ "${backup_erase_after}" -ge "0" ]
    then
      # Purging old backups, only if backup return "OK" status
      #echo "        Giorni da mantenere:" $backup_erase_after
      #echo "         Directory Corrente:" ${PWD}
      #echo "    Lista di tutti i Backup:" *$system*
      #echo "         Lista da mantenere:" $(cat ${file_list_backup_keep})
      #echo "Lista da mantenere cmd find:" $(cat ${file_list_backup_keep_find})
      #echo
      #find -maxdepth 1 -name "*${system}*" $(cat ${file_list_backup_keep_find}) | xargs -r rm -rf
      #find -maxdepth 1 -name "*${system}*" $(cat ${file_list_backup_keep_find}) | xargs -r echo "rm -rf"

      # this is the most dangerous command of synbak ;-)
      cat "${file_list_backup_erase}" | xargs -n1 -r rm -rf
    else
      report_text backup_erase_error
  fi
  cd "${previous_dir}"
}

parse_uri_save_uritmp() {
    uritmp="$(echo ${uritmp} | awk -F "$1" '{print $2}')"
}

parse_uri_check_uri() {
    # check if the uri contain invalid characters
    echo $1 | tr -d [:alnum:] | tr -d "." | tr -d "-" | tr -d ":" | tr -d "/" | tr -d "@" | tr -d "$" | tr -d "~"
}

parse_uri_check_host() {
    # check if the host contain invalid characters
    echo $1 | tr -d [:alnum:] | tr -d "." | tr -d "-" | tr -d ":" | tr -d "/"
}

parse_uri_unset_variables() {
	unset protocol username password host port path
}

parse_uri() {
  local uritmp=$1

  parse_uri_unset_variables

  local uri_delimiter="://"
  local username_delimiter="@"
  local password_delimiter=":"
  local port_delimiter=":"
  local path_delimiter="/"

  # return 0 if no uri is passed to this function
  [ -z "$(echo ${uritmp})" ] && parse_uri_unset_variables && return 0
  # return 1 if the uri doesn't contain the standard uri delimiter
  [ -z "$(echo ${uritmp} | grep ${uri_delimiter})" ] && parse_uri_unset_variables && return 1

  # set the used protocol
  protocol="$(echo ${uritmp} | awk -F "${uri_delimiter}" '{print $1}')" ; parse_uri_save_uritmp "${protocol}${uri_delimiter}"

  # check if the server uri contain a username or password
  if [ -n "$(echo ${uritmp} | grep "${username_delimiter}")" ]
    then
      username="$(echo ${uritmp} | sed "s/${username_delimiter}.*//g")" ; parse_uri_save_uritmp "${username}${username_delimiter}"

      # check if the uri contain the password
      if [ -n "$(echo "${username}" | grep "${password_delimiter}")" ]
        then
          password="$(echo "${username}" | awk -F "${password_delimiter}" '{print $2}')"
          username="$(echo "${username}" | awk -F "${password_delimiter}" '{print $1}')"
      fi

      # stop if no username is set, like wrong written uri with multiple "@" or ":" 
      [ -z "${username}" ] && parse_uri_unset_variables && return 1
  fi

  # stop if the uri contain invalid caracters
  [ -n "$(parse_uri_check_uri "${uritmp}")" ] && parse_uri_unset_variables && return 1

  # check if the uri contain a port number
  if [ -n "$(echo ${uritmp} | grep "${port_delimiter}")" ]
    then
     host="$(echo ${uritmp} | awk -F "${port_delimiter}" '{print $1}')" ; parse_uri_save_uritmp "${host}"
     port="$(echo ${uritmp} | awk -F "${port_delimiter}" '{print $2}' | awk -F "${path_delimiter}" '{print $1}')" ; parse_uri_save_uritmp "${port_delimiter}${port}${path_delimiter}"
     path="$(echo ${uritmp})" ; parse_uri_save_uritmp "${path}"
    else
     host="$(echo ${uritmp} | sed 's/^\/*//g' | awk -F "${path_delimiter}" '{print $1}')" ; parse_uri_save_uritmp "${host}${path_delimiter}"
     path="$(echo ${uritmp})" ; parse_uri_save_uritmp "${path}"
  fi

  # stop if the host contain invalid caracters
  [ -n "$(parse_uri_check_host "${host}")" ] && parse_uri_unset_variables && return 1

  # hide the user password in the uri variable
  if   [[ -n "${password}" && -n "${port}" ]]
    then
     uri="${protocol}${uri_delimiter}${username}${password_delimiter}xxxxxxxx${username_delimiter}${host}${port_delimiter}${port}${path_delimiter}${path}"
  elif [[ -n "${password}" && -z "${port}" ]]
    then
     uri="${protocol}${uri_delimiter}${username}${password_delimiter}xxxxxxxx${username_delimiter}${host}${path_delimiter}${path}"
  elif [ -z "${password}" ]
    then
     uri=$1
  fi

  # print out the parsed fields
  #echo "          URI:" $uri
  #echo "   Protocollo:" $protocol
  #echo "     Username:" $username
  #echo "     Password:" $password
  #echo "         Host:" $host
  #echo "         Port:" $port
  #echo "         Path:" $path
  #echo "URI rimanente:" $uritmp

  unset uritmp
 }


import_backup_remote_variables() {
  local uri="$1"
  parse_uri "$uri"

  # set reusability variables
  backup_remote_uri="$uri"
  backup_remote_protocol="$protocol"
  backup_remote_username="$username"
  backup_remote_password="$password"
  backup_remote_host="$host"
  backup_remote_port="$port"
  backup_remote_path="$path"

  parse_uri_unset_variables
}

# testing reachability of remote netbios host using a samba command
host_check_netbios() {
        local check1="$(smbclient -N -L "$1" >/dev/null 2>&1 ; echo $?)"
        local check2="$(smbclient -d 0 //${host}/${path} $([ -n "${username}" ] && echo "-U ${username}")$([ -n "${password}" ] && echo "%${password}" || echo " -N") -c quit >/dev/null 2>&1 ; echo $?)"
        #echo check1 = $check1
        #echo check2 = $check2
        if [[ "${check1}" = 0 || "${check2}" = 0 ]]
          then
            return 0
          else
            return 1
        fi
	}

# testing reachability of remote host using nc command
host_check_port()    {
	nc -z "$1" "$2" >/dev/null 2>&1
        return $?
	}

# testing reachability of remote host using standard ping
host_check_ping()    {
	local check1=$(ping -c 3 "$1" 2>/dev/stdout 1>/dev/null)
	local check2=$(ping -c 3 "$1" 2>&1 | grep "100%")
	if [[ -n "${check1}" || -n "${check2}" ]]; then return 1 ; else return 0; fi
	}

# function description: verify if the remote server is reachable
# args: 'server uri'
# return: void
check_status_host() {
 local uri="$1"
 parse_uri "$uri"

 case ${protocol} in
	smb|cifs) host_check_netbios "${host}" ;;
	rsync) 	[ -z "${port}" ] && port=873  ; host_check_port "${host}" "${port}" ;;
	ssh)	[ -z "${port}" ] && port=22   ; host_check_port "${host}" "${port}" ;;
	ftp)	[ -z "${port}" ] && port=21   ; host_check_port "${host}" "${port}" ;;
	http)	[ -z "${port}" ] && port=80   ; host_check_port "${host}" "${port}" ;;
	https)	[ -z "${port}" ] && port=443  ; host_check_port "${host}" "${port}" ;;
	ldap)	[ -z "${port}" ] && port=389  ; host_check_port "${host}" "${port}" ;;
	mysql)	[ -z "${port}" ] && port=3306 ; host_check_port "${host}" "${port}" ;;
	oracle)	[ -z "${port}" ] && port=1521 ; host_check_port "${host}" "${port}" ;;
	*)	host_check_ping "${host}" ;;
 esac

 return $?
}


# function description: mount the specified remote share
# args: none (internal synbak variables usage)
mount_host(){
 # Create the temp mount directory
 mkdir -p "${dir_mnt}"

 mount_cifs(){
   mount.cifs //"${backup_remote_host}"/"${backup_remote_path}" "${dir_mnt}" -o ro$([ -n "${backup_remote_username}" ] && echo ",username=$backup_remote_username")$([ -n "${backup_remote_password}" ] && echo ",password=$backup_remote_password" || echo ",guest")
 }

 mount_smb(){
   smbmount //"${backup_remote_host}"/"${backup_remote_path}" "${dir_mnt}" -o ro$([ -n "${backup_remote_username}" ] && echo ",username=$backup_remote_username")$([ -n "${backup_remote_password}" ] && echo ",password=$backup_remote_password" || echo ",guest"),debug=0 | grep -v ^"opts:"
 }

 case "${backup_remote_protocol}" in
        cifs)
                mount_cifs
                ;;
        smb)
                # to fix ad old Red Hat 9 bug when mounting uncomment the following line
                #export LD_ASSUME_KERNEL=2.2.5

		# Revert to standard unix mount.cifs command if smbmount doesn't exist
                if [ $(which smbmount >/dev/null 2>&1 ; echo $?) = 0 ]
                  then
                   mount_smb
                  else
                   mount_cifs
                fi
                ;;
        *)      report_text mount_unsupported
                ;;
 esac

 return $?
}


umount_host() {
 case ${backup_remote_protocol} in
	smb|cifs)
		# Revert to standard unix umount command if smbumount doesn't exist
		if [ $(which smbumount >/dev/null 2>&1 ; echo $?) = 0 ]
                  then
                    smbumount "${dir_mnt}"
                  else
                    umount "${dir_mnt}" 
                fi
		;;
	*)	umount "${dir_mnt}" ;;
 esac
}

# function description: backup verify
# args options: $1 $2 
# args: backup_verify 'type' 'file'
backup_verify() {
  local type="$1"
  shift
  local file="$1"

  case "${type}" in
    tar)
      tar  tvf "$file" | grep ^d
      echo
      ;;
    tgz)
      tar ztvf "$file" | grep ^d
      echo
      ;;
    tbz2)
      tar jtvf "$file" | grep ^d
      echo
      ;;
    tape)
      tar  tvf "$file" | grep ^d
      echo
      ;;
  esac
}


###@@@
###@@@ backup procedures
###@@@

# run the backup method
usr_make_backup() {
  . "${sys_file_method}"
  save_time_end
  save_time_duration
  usr_make_report
}

###@@@
###@@@ report procedures
###@@@

report_run() {
  check_file "${sys_dir_report}/$1/$1.sh"
  if [ $? -eq 0 ]
    then
      local sys_dir_report="${sys_dir_report}/$1"
      local sys_file_report="${sys_dir_report}/$1.sh"
      . "${sys_file_report}"
  fi
}

# run the backup reports
usr_make_report() {
  if [[ ${status_host} = 1 && "${report_remote_uri_down}" = "no" ]]
    then
      return 0
    else
      for report in $(find "${sys_dir_report}" -maxdepth 1 -type d | sed -e 's/^.\///' | grep -wv ^"." | grep -v "${sys_dir_report}"$)
        do
          check_file ${report}/$(basename ${report}).sh
          if [ $? -eq 0 ]
            then
              report_run $(basename ${report}) backup 
          fi
        done
  fi
}


