#!/bin/sh
#
# Copyright (C) 2003, 2005 InitZero S.rl.
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software Foundation,
# Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
#

# import config fields
file_config_field_import ${usr_file_conf} report_html			bolean	notnull
file_config_field_import ${usr_file_conf} report_html_on_errors	bolean	null
file_config_field_import ${usr_file_conf} report_html_uri		uri	notnull
file_config_field_import ${usr_file_conf} report_html_destination	text	notnull
file_config_field_import ${usr_file_conf} report_html_logo		bolean	null
file_config_field_import ${usr_file_conf} report_html_logo_image	text	null
file_config_field_import ${usr_file_conf} report_html_logo_link		uri	null
check_dir ${report_html_destination} ; [ $? -ne 0 ] && return 1; # check if the destination html report directory or device exist


# init local variables useful to this report
local year=$(show_time_human   ${time_begin} year)
local month=$(show_time_human  ${time_begin} month)
local day=$(show_time_human    ${time_begin} day)
local hour=$(show_time_human   ${time_begin} hour)
local minute=$(show_time_human ${time_begin} minute)
local second=$(show_time_human ${time_begin} second)
local file_cnt_ok="cnt.ok"
local file_cnt_errors="cnt.err"

local report_html_destination_year="${report_html_destination}/${year}"
local report_html_destination_month="${report_html_destination_year}/${month}"
local report_html_destination_day="${report_html_destination_month}/${day}"
local report_html_destination_backup="${report_html_destination_day}/${time_begin}.$RANDOM"
local report_html_file_index="index.html"
local report_html_file_css="synbak.css"
local report_html_file_logo="initzero-logo.jpg"
local report_rss_file_index="synbak.xml"


report_html() {
  case $1 in
	page_title)	msg "Synbak :: HTML Backup Report :: Made on " ; echo -n "$(show_time_human $(show_time_unix) full)" ;;
	previous_page)	msg "Go to previous page" ;;
	backup_made)	msg "Backups made by Synbak v\${synbak_version} on" ;;
	total)		msg "Year" ;;
	year)		msg "Month" ;;
	month)		msg "Day" ;;
	synbak_server)	msg "Server" ;;
	system)		msg "System" ;;
	method)		msg "Method" ;;
	method_option)	msg "Option" ;;
	method_type)	msg "Type" ;;
	time_begin)	msg "Begin" ;;
	time_end)	msg "End" ;;
	time_duration)	msg "Duration" ;;
	size_backup)	msg "Bak. Size" ;;
	size_destination)msg "Tot. Size" ;;
	speed_backup)	msg "Speed" ;;
	status_backup)	msg "Result" ;;
	ok)		msg "OK" ;;
	error)		msg "ERROR" ;;
	oks)		msg "Ok" ;;
	errors)		msg "Errors" ;;
	totals)		msg "Totals" ;;
	rebuild_indexes)msg "Rebuilding indexes:" ;;
	unknow)		msg "unknow" ;;
	dir_unwritable)	msg "ERROR: The html destination directory '\${report_html_destination_backup}' is unwritable" ;;
  esac
}

read_file(){
  if [ -f "$1" ]
    then
	cat "$1"
    else
	report_html unknow
	return 1
  fi
}

archive_logs() {
  echo "<pre>"			>	$1/${report_html_file_index}
  read_file ${file_log_output} >>	$1/${report_html_file_index}

  cp -f ${file_synbak_server}	$1/
  cp -f ${file_system}		$1/
  cp -f ${file_method}		$1/
  cp -f ${file_method_option}	$1/
  cp -f ${file_method_type}	$1/
  cp -f ${file_status_backup}	$1/
  cp -f ${file_size_backup}	$1/
  cp -f ${file_size_destination}	$1/
  cp -f ${file_time_begin}	$1/
  cp -f ${file_time_end}	$1/
}


  html_header() {
   css_mng() {
     case $1 in
       total)	echo "${report_html_file_css}" ;;
       year)	echo "../${report_html_file_css}" ;;
       month)	echo "../../${report_html_file_css}" ;;
       day)	echo "../../../${report_html_file_css}" ;;
     esac
   }

   logo_mng() {
     case $1 in
       total) if [ "${report_html_logo}" = "yes" ]; then echo "<a href=${report_html_logo_link}>"; 
              if [ -n "${report_html_logo_image}" ]; then echo "<img border=0 src=${report_html_logo_image}></a>";
                                                     else echo "<img border=0 src=${report_html_file_logo}></a>"; fi; fi ;;
       year)  if [ "${report_html_logo}" = "yes" ]; then echo "<a href=${report_html_logo_link}>"; 
              if [ -n "${report_html_logo_image}" ]; then echo "<img border=0 src=${report_html_logo_image}></a>";
                                                     else echo "<img border=0 src=../${report_html_file_logo}></a>"; fi; fi ;;
       month) if [ "${report_html_logo}" = "yes" ]; then echo "<a href=${report_html_logo_link}>"; 
              if [ -n "${report_html_logo_image}" ]; then echo "<img border=0 src=${report_html_logo_image}></a>";
                                                     else echo "<img border=0 src=../../${report_html_file_logo}></a>"; fi; fi ;;
       day)   if [ "${report_html_logo}" = "yes" ]; then echo "<a href=${report_html_logo_link}>"; 
              if [ -n "${report_html_logo_image}" ]; then echo "<img border=0 src=${report_html_logo_image}></a>";
                                                     else echo "<img border=0 src=../../../${report_html_file_logo}></a>"; fi; fi ;;
     esac
   }

    echo "<html>
    <head>
      <title>$(report_html page_title)</title>
      <link rel=\"alternate\" type=\"application/rss+xml\" title=\"Synbak RSS\" href=\"${report_html_uri}/${report_rss_file_index}\">
      <meta name=\"copyright\" content=\"${synbak_copyright}\" />
      <meta name=\"author\" content=\"${synbak_author}\" />
      <link href=\"$(css_mng $1)\" rel=\"stylesheet\">
    </head>
    <body>
      <div align=center>
        $(logo_mng $1)
        <p><h3><br>${synbak_description}<br>${synbak_copyright}</h3></p>"
  } 

  # backup status counter function
  status_cnt(){
    local adir=$1
    local type=$2
    case $type in
	ok)
		read_file ${adir}/${file_cnt_ok} 2>/dev/null
		;;
	errors)
		read_file ${adir}/${file_cnt_errors} 2>/dev/null
		;;
	totals)
		let cnt=$(read_file ${adir}/${file_cnt_ok})+$(read_file ${adir}/${file_cnt_errors}) 2>/dev/null
                echo ${cnt}
                unset cnt
		;;
	cnt_days)
		find ${adir} -name $(basename ${file_status_backup}) | xargs -r cat | grep ^"0" | wc -l | awk '{print $1}' > ${adir}/${file_cnt_ok}     2>/dev/null
		find ${adir} -name $(basename ${file_status_backup}) | xargs -r cat | grep ^"1" | wc -l | awk '{print $1}' > ${adir}/${file_cnt_errors} 2>/dev/null
		;;
	cnt_all)
                cnt_func() {
		  local cnt=0
                  # $1 is a directory
                  # $2 is the dest file name
                  for file in $(find $1 -maxdepth 2 -type f -name $2 | sed -e 's/^.\///' | grep -wv ^"." | grep -v "$1/$2"$ | sort -r)
                    do
		      let cnt=$(read_file $file)+$cnt
		    done
		  echo "${cnt}"
                  unset cnt
                }
                
                cnt_func ${adir} ${file_cnt_ok}     > ${adir}/${file_cnt_ok}     2>/dev/null
                cnt_func ${adir} ${file_cnt_errors} > ${adir}/${file_cnt_errors} 2>/dev/null
		;;
    esac
  }

  # $1 = index description name
  # $2 = directory to indicize
  html_body() {
    make_page_table_list(){

        # don't count the status of backups in the total page, is not useful
        [ "$2" != "total" ] && status_cnt $1 cnt_all

        for dir in $(find $1 -maxdepth 1 -type d | sed -e 's/^.\///' | grep -wv ^"." | grep -v "$1"$ | sort -r)
          do
            local rdir=$(basename ${dir})
            local desc=${rdir}
            local oks="$(status_cnt $dir ok)"
            local errors=$(status_cnt $dir errors)
            echo "     <tr>
         <td class='sb_cel_body2' style='text-align: center;'><a href=\"${rdir}/${report_html_file_index}\"><b>${desc}</b></a></td>
         <td class='sb_cel_body'  style='text-align: center;'>$([ "${oks}" = "0" ] && echo "<span>&nbsp;" || echo "<span class=TextOks>${oks}")</span></td>
         <td class='sb_cel_body'  style='text-align: center;'>$([ "${errors}" = "0" ] && echo "<span>&nbsp;" || echo "<span class=TextErrors>${errors}")</span></td>
         <td class='sb_cel_body3' style='text-align: center;'><span class=TextTotals>$(status_cnt $dir totals)</span></td>
       </tr>"
          done

    }

   make_page_table_day(){
     # count all backup status on current day
     status_cnt $1 cnt_days
     local row=$(find $1 -maxdepth 1 -type d | sed -e 's/^.\///' | grep -wv ^"." | grep -v "$1"$ | wc -l | awk '{print $1}')

     # print a row for any backups found in the current day
     for dir in $(find $1 -maxdepth 1 -type d | sed -e 's/^.\///' | grep -wv ^"." | grep -v "$1"$ | sort -r)
       do
            local rdir=$(basename ${dir})
            local desc=${rdir}

            if [ "$(read_file ${dir}/$(basename ${file_status_backup}))" = "0" ]
              then
		local cel_class_body=sb_cel_body_ok
		local cel_class_body2=sb_cel_body2_ok
		local cel_class_status=sb_cel_status_ok
                local status_backup=$(report_html ok)
              else
		local cel_class_body=sb_cel_body_error
		local cel_class_body2=sb_cel_body2_error
		local cel_class_status=sb_cel_status_error
                local status_backup=$(report_html error)
            fi

            local method_option=$(read_file ${dir}/$(basename ${file_method_option}))

            echo "      <tr>
            <td class='${cel_class_body}'  style='text-align: left;'><span class='TextRow'>$row)&nbsp;</span></td>
            <td class='${cel_class_body2}' style='text-align: left;'><b>$(read_file ${dir}/$(basename ${file_system}))</b></td>
            <td class='${cel_class_body2}' style='text-align: left;'>$(read_file ${dir}/$(basename ${file_method}))</td>
            <td class='${cel_class_body2}' style='text-align: left;'>$([ -n "${method_option}" ] && echo "${method_option}" || echo "&nbsp;")</td>
            <td class='${cel_class_body2}' style='text-align: left;'>$(read_file ${dir}/$(basename ${file_method_type}))</td>
            <td class='${cel_class_body}'  style='text-align: center;'>$(show_time_human $(read_file ${dir}/$(basename ${file_time_begin})))</td>
            <td class='${cel_class_body}'  style='text-align: center;'>$(show_time_human $(read_file ${dir}/$(basename ${file_time_end})))</td>
            <td class='${cel_class_body}'  style='text-align: center;'>$(show_time_duration_human $(read_file ${dir}/$(basename ${file_time_begin})) $(read_file ${dir}/$(basename ${file_time_end})))</td>
            <td class='${cel_class_body}'  style='text-align: right;'>$(show_size $(read_file ${dir}/$(basename ${file_size_backup})))</b></td>
            <td class='${cel_class_body}'  style='text-align: right;'>$(show_speed $(read_file ${dir}/$(basename ${file_size_backup})) $(read_file ${dir}/$(basename ${file_time_begin})) $(read_file ${dir}/$(basename ${file_time_end})))</b></td>
            <td class='${cel_class_body}'  style='text-align: right;'>$(show_size $(read_file ${dir}/$(basename ${file_size_destination})))</b></td>
            <td class=${cel_class_status}  style='text-align: center;'><a href=\"${rdir}/${report_html_file_index}\"><b>${status_backup}</b></a></td>
            </tr>"
            let row=${row}-1
       done
   }

     case $1 in
       total|year|month)
	echo "      <table class='sb_table'>
	<td class='sb_cel_head' style='text-align: center;'><b>$(report_html $1)</b></td>
	<td class='sb_cel_head' style='text-align: center;'><b>$(report_html oks)</b></td>
	<td class='sb_cel_head' style='text-align: center;'><b>$(report_html errors)</b></td>
	<td class='sb_cel_head' style='text-align: center;'><b>$(report_html totals)</b></td>
	$(make_page_table_list $2 $1)
	</table>"
	;;
       day)
	echo "<p>$(report_html backup_made) $(basename ${year})/$(basename ${month})/$(basename ${day})</p>
	<table class='sb_table'>
	<td class='sb_cel_head' style='text-align: center;'><b>#</b></td>
	<td class='sb_cel_head' style='text-align: center;'><b>$(report_html system)</b></td>
	<td class='sb_cel_head' style='text-align: center;'><b>$(report_html method)</b></td>
	<td class='sb_cel_head' style='text-align: center;'><b>$(report_html method_option)</b></td>
	<td class='sb_cel_head' style='text-align: center;'><b>$(report_html method_type)</b></td>
	<td class='sb_cel_head' style='text-align: center;'><b>$(report_html time_begin)</b></td>
	<td class='sb_cel_head' style='text-align: center;'><b>$(report_html time_end)</b></td>
	<td class='sb_cel_head' style='text-align: center;'><b>$(report_html time_duration)</b></td>
	<td class='sb_cel_head' style='text-align: center;'><b>$(report_html size_backup)</b></td>
	<td class='sb_cel_head' style='text-align: center;'><b>$(report_html speed_backup)</b></td>
	<td class='sb_cel_head' style='text-align: center;'><b>$(report_html size_destination)</b></td>
	<td class='sb_cel_head' style='text-align: center;'><b>$(report_html status_backup)</b></td>
	$(make_page_table_day $2 $1)
	</table>"
	;;
     esac
  }

  html_footer() {
    echo "$([ "$1" != "total" ] && echo "    <p><h4><a href=\"../\">$(report_html previous_page)</a></h4></p>")
     </div>
     </body>
</html>"
  }

  rss_header() {
echo "<?xml version=\"1.0\" encoding=\"UTF-8\"?>
<rss version='2.0'>
<channel>
<title>Synbak :: RSS Backup Report</title>
<link>${report_html_uri}/${year}/${month}/${day}</link>
<copyright>${synbak_copyright}</copyright>
<author>${synbak_author}</author>
<generator>${synbak_description} v.${synbak_version}-${synbak_version_date}</generator>
<pubDate>$(show_time_human $(show_time_unix) rss)</pubDate>
<lastBuildDate>$(show_time_human $(show_time_unix) rss)</lastBuildDate>
<ttl>60</ttl>
<description>$(report_html backup_made) $(echo -n "$(show_time_human $(show_time_unix) year)/$(show_time_human $(show_time_unix) month)/$(show_time_human $(show_time_unix) day)")</description>
<image>
  <title>${synbak_description}</title>
  <link>${report_html_uri}</link>
  <uri>${report_html_uri}/${report_html_file_logo}</uri>
</image>"
  }

  rss_body() {
     local tdir="$1"
     # count all backup status on current day
     status_cnt ${cdir} cnt_days

     # print a row for any backups found in the current day
     for dir in $(find ${tdir} -maxdepth 1 -type d | sed -e 's/^.\///' | grep -wv ^"." | grep -v "${tdir}"$ | sort -r)
       do
            local rdir=$(basename ${dir})
            local desc=${rdir}
            local status_color="#000000"

            if [ $(read_file ${dir}/$(basename ${file_status_backup})) = 0 ]
              then
		local status_color="#00cc00"
                local status_backup="$(report_html ok)"
              else
		local status_color="#ff0000"
                local status_backup="$(report_html error)"
            fi

	echo "<item>
  <title>B:[$(show_time_human $(read_file ${dir}/$(basename ${file_time_begin})))] S:[$(read_file ${dir}/$(basename ${file_system}))] M:[$(read_file ${dir}/$(basename ${file_method}))] R:[${status_backup}]</title>
  <link>${report_html_uri}/${year}/${month}/${day}/${rdir}/${report_html_file_index}</link>
  <author>${report_info_support}</author>
  <description>
&lt;i&gt;$(report_html system)&lt;/i&gt;:[&lt;b&gt;$(read_file ${dir}/$(basename ${file_system}))&lt;/b&gt;]
&lt;i&gt;$(report_html method)&lt;/i&gt;:[&lt;b&gt;$(read_file ${dir}/$(basename ${file_method}))&lt;/b&gt;]
&lt;i&gt;$(report_html method_option)&lt;/i&gt;:[&lt;b&gt;$(read_file ${dir}/$(basename ${file_method_option}))&lt;/b&gt;]
&lt;i&gt;$(report_html method_type)&lt;/i&gt;:[&lt;b&gt;$(read_file ${dir}/$(basename ${file_method_type}))&lt;/b&gt;]
&lt;i&gt;$(report_html time_duration)&lt;/i&gt;:[&lt;b&gt;$(show_time_duration_human $(read_file ${dir}/$(basename ${file_time_begin})) $(read_file ${dir}/$(basename ${file_time_end})))&lt;/b&gt;]
&lt;i&gt;$(report_html size_backup)&lt;/i&gt;:[&lt;b&gt;$(show_size $(read_file ${dir}/$(basename ${file_size_backup})))&lt;/b&gt;]
&lt;i&gt;$(report_html speed_backup)&lt;/i&gt;:[&lt;b&gt;$(show_speed $(read_file ${dir}/$(basename ${file_size_backup})) $(read_file ${dir}/$(basename ${file_time_begin})) $(read_file ${dir}/$(basename ${file_time_end})))&lt;/b&gt;]
&lt;i&gt;$(report_html status_backup)&lt;/i&gt;:[&lt;b&gt;&lt;font color=$status_color&gt;${status_backup}&lt;/font&gt;&lt;/b&gt;]
  </description>
  <pubDate>$(show_time_human $(read_file ${dir}/$(basename ${file_time_begin})) rss)</pubDate>
</item>"
       done
}

#&lt;i&gt;$(report_html synbak_server)&lt;/i&gt;:[&lt;b&gt;$(read_file ${dir}/$(basename ${file_synbak_server}))&lt;/b&gt;]
#&lt;i&gt;$(report_html time_begin)&lt;/i&gt;:[&lt;b&gt;$(show_time_human $(read_file ${dir}/$(basename ${file_time_begin})))&lt;/b&gt;]
#&lt;i&gt;$(report_html time_end)&lt;/i&gt;:[&lt;b&gt;$(show_time_human $(read_file ${dir}/$(basename ${file_time_end})))&lt;/b&gt;]


  rss_footer() {
echo "</channel>
</rss>"
  }

make_rss_index() {
  rss_header	$1 $2
  rss_body	$2 $1
  rss_footer	$1 $2
}


report_make() {
  mkdir -p "${report_html_destination_backup}"

  if [ $? -ne 0 ] 
    then
      #report_html dir_unwritable
      return 1;
    else
      archive_logs "${report_html_destination_backup}"
  fi

  # Copy the default system css file to synbak html report root 
  cp "${sys_dir_report}"/"${report_html_file_css}" "${report_html_destination}"
  cp "${sys_dir_report}"/"${report_html_file_logo}" "${report_html_destination}"

  make_html_index day	"${report_html_destination_day}"	> "${report_html_destination_day}"/"${report_html_file_index}"
  make_html_index month	"${report_html_destination_month}"	> "${report_html_destination_month}"/"${report_html_file_index}"
  make_html_index year	"${report_html_destination_year}"	> "${report_html_destination_year}"/"${report_html_file_index}"
  make_html_index total	"${report_html_destination}"		> "${report_html_destination}"/"${report_html_file_index}"

  make_rss_index  day	"${report_html_destination_day}"	> "${report_html_destination}"/"${report_rss_file_index}"
}


make_html_index() {
  html_header	$1 $2
  html_body	$1 $2
  html_footer	$1 $2
}

# rebuild all indexes
rebuild_indexes() {
  for year in $(find "${report_html_destination}" -maxdepth 1 -type d | sed -e 's/^.\///' | grep -wv ^"." | grep -v "${report_html_destination}"$ | sort -r)
   do
    for month in $(find "${year}" -maxdepth 1 -type d | sed -e 's/^.\///' | grep -wv ^"." | grep -v "${year}"$ | sort -r)
     do
      for day in $(find "${month}" -maxdepth 1 -type d | sed -e 's/^.\///' | grep -wv ^"." | grep -v "${month}"$ | sort -r)
       do
        make_html_index day "${day}" > "${day}"/"${report_html_file_index}"
       done
      make_html_index month "${month}" > "${month}"/"${report_html_file_index}"
     done
    make_html_index year "${year}" > "${year}"/"${report_html_file_index}"
   done

  make_html_index total "${report_html_destination}"		> "${report_html_destination}"/"${report_html_file_index}"
  make_rss_index  day   "${report_html_destination_day}"        > "${report_html_destination}"/"${report_rss_file_index}"
}

#echo $@

# parse the report input command and make the report
case $2 in
	ria) # comands specific to this method
		report_html rebuild_indexes
                save_time_begin_step
                report_text report_step
		rebuild_indexes > "${file_log_errors}" 2>&1
                save_time_end_step
                check_status_step
		;;
	backup)
		if [[ "${report_html}" = "yes" && "$1" = "html" ]]
		  then
		    #echo "Normal Reports run"
                    save_time_begin_step
                    report_text report_step
		    report_make > "${file_log_errors}" 2>&1
                    save_time_end_step
                    check_status_step
		  else
		    check_status_backup 2>&1 > /dev/null
		    if [[ $? -eq 1 && "${report_html_on_errors}" = "yes" && "$1" = "html" ]]
		      then
		        # echo "forcing html report errors only"
                        save_time_begin_step
                        report_text report_step
	                report_make > "${file_log_errors}" 2>&1
                        save_time_end_step
                        check_status_step
		    fi
		fi
		;;
	*)
		local extra_option_right="ria"
		local current_option="$2"
		[ -z "$2" ] && report_text extra_option_required && exit 1
		[ "$2" != "ria" ] && report_text extra_option_wrong && exit 1
		;;
esac


